@@ -1,5 +1,5 @@
 # Copyright (c) 2017 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 from enum import IntEnum
 
@@ -55,33 +55,30 @@ class Backend(PluginObject):
     ##   \brief Start the backend / engine.
     #   Runs the engine, this is only called when the socket is fully opened & ready to accept connections
     def startEngine(self):
-        try:
-            command = self.getEngineCommand()
-            if not command:
-                self._createSocket()
-                return
+        command = self.getEngineCommand()
+        if not command:
+            self._createSocket()
+            return
 
-            if not self._backend_log_max_lines:
-                self._backend_log = []
+        if not self._backend_log_max_lines:
+            self._backend_log = []
 
-            # Double check that the old process is indeed killed.
-            if self._process is not None:
-                self._process.terminate()
-                Logger.log("d", "Engine process is killed. Received return code %s", self._process.wait())
+        # Double check that the old process is indeed killed.
+        if self._process is not None:
+            self._process.terminate()
+            Logger.log("d", "Engine process is killed. Received return code %s", self._process.wait())
 
-            self._process = self._runEngineProcess(command)
-            if self._process is None: #Failed to start engine.
-                return
-            Logger.log("i", "Started engine process: %s", self.getEngineCommand()[0])
-            self._backendLog(bytes("Calling engine with: %s\n" % self.getEngineCommand(), "utf-8"))
-            t = threading.Thread(target = self._storeOutputToLogThread, args = (self._process.stdout,))
-            t.daemon = True
-            t.start()
-            t = threading.Thread(target = self._storeStderrToLogThread, args = (self._process.stderr,))
-            t.daemon = True
-            t.start()
-        except FileNotFoundError as e:
-            Logger.log("e", "Unable to find backend executable: %s", self.getEngineCommand()[0])
+        self._process = self._runEngineProcess(command)
+        if self._process is None:  # Failed to start engine.
+            return
+        Logger.log("i", "Started engine process: %s", self.getEngineCommand()[0])
+        self._backendLog(bytes("Calling engine with: %s\n" % self.getEngineCommand(), "utf-8"))
+        t = threading.Thread(target = self._storeOutputToLogThread, args = (self._process.stdout,))
+        t.daemon = True
+        t.start()
+        t = threading.Thread(target = self._storeStderrToLogThread, args = (self._process.stderr,))
+        t.daemon = True
+        t.start()
 
     def close(self):
         if self._socket:
@@ -90,7 +87,11 @@ class Backend(PluginObject):
             self._socket.close()
 
     def _backendLog(self, line):
-        Logger.log('d', "[Backend] " + str(line, encoding="utf-8").strip())
+        try:
+            line_str = line.decode("utf-8")
+        except UnicodeDecodeError:
+            line_str = line.decode("latin1") #Latin-1 as a fallback since it can never give decoding errors. All characters are 1 byte.
+        Logger.log("d", "[Backend] " + line_str.strip())
         self._backend_log.append(line)
 
     ##  Get the logging messages of the backend connection.
@@ -142,6 +143,8 @@ class Backend(PluginObject):
             return subprocess.Popen(command_list, stdin = subprocess.DEVNULL, stdout = subprocess.PIPE, stderr = subprocess.PIPE, **kwargs)
         except PermissionError:
             Logger.log("e", "Couldn't start back-end: No permission to execute process.")
+        except FileNotFoundError:
+            Logger.logException("e", "Unable to find backend executable: %s", command_list[0])
 
     def _storeOutputToLogThread(self, handle):
         while True:
@@ -230,7 +233,9 @@ class Backend(PluginObject):
         if Platform.isWindows():
             # On Windows, the Protobuf DiskSourceTree does stupid things with paths.
             # So convert to forward slashes here so it finds the proto file properly.
-            protocol_file = protocol_file.replace("\\", "/")
+            # Using sys.getfilesystemencoding() avoid the application crashing if it is
+            # installed on a path with non-ascii characters GitHub issue #3907
+            protocol_file = protocol_file.replace("\\", "/").encode(sys.getfilesystemencoding())
 
         if not self._socket.registerAllMessageTypes(protocol_file):
             Logger.log("e", "Could not register Uranium protocol messages: %s", self._socket.getLastError())
