@@ -1,8 +1,7 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-
 from UM.Scene.Scene import Scene
-from UM.Event import Event, MouseEvent, ToolEvent, ViewEvent
+from UM.Event import Event, KeyEvent, MouseEvent, ToolEvent, ViewEvent
 from UM.Signal import Signal, signalemitter
 from UM.Logger import Logger
 from UM.PluginRegistry import PluginRegistry
@@ -11,38 +10,44 @@ from UM.PluginRegistry import PluginRegistry
 from UM.View.View import View
 from UM.Stage import Stage
 from UM.InputDevice import InputDevice
-from typing import Optional, Dict
+from typing import cast, Optional, Dict, Union
 from UM.Math.Vector import Vector
 
-##      Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
+MYPY = False
+if MYPY:
+    from UM.Application import Application
+    from UM.Tool import Tool
+
+
+##  Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
 #
-#       The different types of views / tools / inputs are defined by plugins.
-#       \sa View
-#       \sa Tool
-#       \sa Scene
+#   The different types of views / tools / inputs are defined by plugins.
+#   \sa View
+#   \sa Tool
+#   \sa Scene
 @signalemitter
 class Controller:
-    def __init__(self, application):
+    def __init__(self, application: "Application") -> None:
         super().__init__()  # Call super to make multiple inheritance work.
 
         self._scene = Scene()
         self._application = application
         self._is_model_rendering_enabled = True
 
-        self._active_view = None
-        self._views = {}
+        self._active_view = None  # type: Optional[View]
+        self._views = {}  # type: Dict[str, View]
 
-        self._active_tool = None
+        self._active_tool = None  # type: Optional[Tool]
         self._tool_operation_active = False
-        self._tools = {}
-        self._camera_tool = None
-        self._selection_tool = None
-        self._tools_enabled = True
+        self._tools = {}  # type: Dict[str, Tool]
+        self._camera_tool = None #type: Optional[Tool]
+        self._selection_tool = None #type: Optional[Tool]
+        self._tools_enabled = True #type: bool
 
-        self._active_stage = None
-        self._stages = {}
+        self._active_stage = None #type: Optional[Stage]
+        self._stages = {} #type: Dict[str, Stage]
 
-        self._input_devices = {}
+        self._input_devices = {} #type: Dict[str, InputDevice]
 
         PluginRegistry.addType("stage", self.addStage)
         PluginRegistry.addType("view", self.addView)
@@ -51,13 +56,13 @@ class Controller:
 
     ##  Get the application.
     #   \returns Application \type {Application}
-    def getApplication(self):
+    def getApplication(self) -> "Application":
         return self._application
 
     ##  Add a view by name if it"s not already added.
     #   \param name \type{string} Unique identifier of view (usually the plugin name)
     #   \param view \type{View} The view to be added
-    def addView(self, view: View):
+    def addView(self, view: View) -> None:
         name = view.getPluginId()
         if name not in self._views:
             self._views[name] = view
@@ -88,7 +93,7 @@ class Controller:
 
     ##  Set the currently active view.
     #   \param name \type{string} The name of the view to set as active
-    def setActiveView(self, name: str):
+    def setActiveView(self, name: str) -> None:
         Logger.log("d", "Setting active view to %s", name)
         try:
             if self._active_view:
@@ -105,13 +110,13 @@ class Controller:
         except Exception as e:
             Logger.logException("e", "An exception occurred while switching views: %s", str(e))
 
-    def enableModelRendering(self):
+    def enableModelRendering(self) -> None:
         self._is_model_rendering_enabled = True
 
-    def disableModelRendering(self):
+    def disableModelRendering(self) -> None:
         self._is_model_rendering_enabled = False
 
-    def isModelRenderingEnabled(self):
+    def isModelRenderingEnabled(self) -> bool:
         return self._is_model_rendering_enabled
 
     ##  Emitted when the list of views changes.
@@ -123,7 +128,7 @@ class Controller:
     ##  Add a stage by name if it's not already added.
     #   \param name \type{string} Unique identifier of stage (usually the plugin name)
     #   \param stage \type{Stage} The stage to be added
-    def addStage(self, stage: Stage):
+    def addStage(self, stage: Stage) -> None:
         name = stage.getPluginId()
         if name not in self._stages:
             self._stages[name] = stage
@@ -151,11 +156,19 @@ class Controller:
 
     ##  Set the currently active stage.
     #   \param name \type{string} The name of the stage to set as active
-    def setActiveStage(self, name: str):
-        Logger.log("d", "Setting active stage to %s", name)
+    def setActiveStage(self, name: str) -> None:
         try:
-            self._active_stage = self._stages[name]
-            self.activeStageChanged.emit()
+            # Don't actually change the stage if it is the current selected one.
+            if self._active_stage != self._stages[name]:
+                previous_stage = self._active_stage
+                Logger.log("d", "Setting active stage to %s", name)
+                self._active_stage = self._stages[name]
+
+                # If there is no error switching stages, then finish first the previous stage (if it exists) and start the new stage
+                if previous_stage is not None:
+                    previous_stage.onStageDeselected()
+                self._active_stage.onStageSelected()
+                self.activeStageChanged.emit()
         except KeyError:
             Logger.log("e", "No stage named %s found", name)
         except Exception as e:
@@ -169,7 +182,7 @@ class Controller:
 
     ##  Add an input device (e.g. mouse, keyboard, etc) if it's not already added.
     #   \param device The input device to be added
-    def addInputDevice(self, device: InputDevice):
+    def addInputDevice(self, device: InputDevice) -> None:
         name = device.getPluginId()
         if name not in self._input_devices:
             self._input_devices[name] = device
@@ -190,15 +203,15 @@ class Controller:
     ##  Remove an input device from the list of input devices.
     #   Does nothing if the input device is not in the list.
     #   \param name \type{string} The name of the device to remove.
-    def removeInputDevice(self, name: str):
+    def removeInputDevice(self, name: str) -> None:
         if name in self._input_devices:
             self._input_devices[name].event.disconnect(self.event)
             del self._input_devices[name]
 
     ##  Request tool by name. Returns None if no view is found.
     #   \param name \type{string} Unique identifier of tool (usually the plugin name)
-    #   \return tool \type{Tool} if name was found, none otherwise.
-    def getTool(self, name: str):
+    #   \return tool \type{Tool} if name was found, None otherwise.
+    def getTool(self, name: str) -> Optional["Tool"]:
         try:
             return self._tools[name]
         except KeyError:  # No such tool
@@ -207,12 +220,12 @@ class Controller:
 
     ##  Get all tools
     #   \return tools \type{dict}
-    def getAllTools(self):
+    def getAllTools(self) -> Dict[str, "Tool"]:
         return self._tools
 
     ##  Add a Tool (transform object, translate object) if its not already added.
     #   \param tool \type{Tool} Tool to be added
-    def addTool(self, tool):
+    def addTool(self, tool: "Tool") -> None:
         name = tool.getPluginId()
         if name not in self._tools:
             self._tools[name] = tool
@@ -222,12 +235,12 @@ class Controller:
         else:
             Logger.log("w", "%s was already added to tool list. Unable to add it again.", name)
 
-    def _onToolOperationStarted(self, tool):
+    def _onToolOperationStarted(self, tool: "Tool") -> None:
         if not self._tool_operation_active:
             self._tool_operation_active = True
             self.toolOperationStarted.emit(tool)
 
-    def _onToolOperationStopped(self, tool):
+    def _onToolOperationStopped(self, tool: "Tool") -> None:
         if self._tool_operation_active:
             self._tool_operation_active = False
             self.toolOperationStopped.emit(tool)
@@ -239,19 +252,19 @@ class Controller:
 
     ##  Request active tool. Returns None if there is no active tool
     #   \return Tool \type{Tool} if an tool is active, None otherwise.
-    def getActiveTool(self):
+    def getActiveTool(self) -> Optional["Tool"]:
         return self._active_tool
 
     ##  Set the current active tool.
     #   The tool can be set by name of the tool or directly passing the tool object.
     #   \param tool \type{Tool} or \type{string}
-    def setActiveTool(self, tool):
+    def setActiveTool(self, tool: Optional[Union["Tool", str]]):
         from UM.Tool import Tool
         if self._active_tool:
             self._active_tool.event(ToolEvent(ToolEvent.ToolDeactivateEvent))
 
         if isinstance(tool, Tool) or tool is None:
-            new_tool = tool
+            new_tool = cast(Optional[Tool], tool)
         else:
             new_tool = self.getTool(tool)
 
@@ -312,6 +325,7 @@ class Controller:
             return
 
         if self._tools and event.type == Event.KeyPressEvent:
+            event = cast(KeyEvent, event)
             from UM.Scene.Selection import Selection  # Imported here to prevent a circular dependency.
             if Selection.hasSelection():
                 for key, tool in self._tools.items():
@@ -328,8 +342,10 @@ class Controller:
         if self._active_view:
             self._active_view.event(event)
 
-        if event.type == Event.MouseReleaseEvent and MouseEvent.RightButton in event.buttons:
-            self.contextMenuRequested.emit(event.x, event.y)
+        if event.type == Event.MouseReleaseEvent:
+            event = cast(MouseEvent, event)
+            if MouseEvent.RightButton in event.buttons:
+                self.contextMenuRequested.emit(event.x, event.y)
 
     contextMenuRequested = Signal()
 
@@ -339,16 +355,16 @@ class Controller:
     #   \param tool \type{Tool} or \type{string}
     #   \sa setSelectionTool
     #   \sa setActiveTool
-    def setCameraTool(self, tool):
+    def setCameraTool(self, tool: Union["Tool", str]):
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
-            self._camera_tool = tool
+            self._camera_tool = cast(Optional[Tool], tool)
         else:
             self._camera_tool = self.getTool(tool)
 
     ##  Get the camera tool (if any)
     #   \returns camera tool (or none)
-    def getCameraTool(self):
+    def getCameraTool(self) -> Optional["Tool"]:
         return self._camera_tool
 
     ##  Set the tool used for performing selections.
@@ -357,33 +373,35 @@ class Controller:
     #   \param tool \type{Tool} or \type{string}
     #   \sa setCameraTool
     #   \sa setActiveTool
-    def setSelectionTool(self, tool):
+    def setSelectionTool(self, tool: Union[str, "Tool"]):
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
-            self._selection_tool = tool
+            self._selection_tool = cast(Optional[Tool], tool)
         else:
             self._selection_tool = self.getTool(tool)
 
-    def getToolsEnabled(self):
+    def getToolsEnabled(self) -> bool:
         return self._tools_enabled
 
-    def setToolsEnabled(self, enabled):
+    def setToolsEnabled(self, enabled: bool) -> None:
         self._tools_enabled = enabled
 
     # Rotate camera view according defined angle
-    def rotateView(self, coordinate = "x", angle = 0):
+    def rotateView(self, coordinate: str = "x", angle: int = 0) -> None:
         camera = self._scene.getActiveCamera()
-        self._camera_tool.setOrigin(Vector(0, 100, 0))
+        if not camera:
+            return
+        self._camera_tool.setOrigin(Vector(0, 100, 0)) #type: ignore
         if coordinate == "home":
             camera.setPosition(Vector(0, 0, 700))
             camera.setPerspective(True)
             camera.lookAt(Vector(0, 100, 100))
-            self._camera_tool.rotateCam(0, 0)
+            self._camera_tool.rotateCam(0, 0) #type: ignore
         elif coordinate == "3d":
             camera.setPosition(Vector(-750, 600, 700))
             camera.setPerspective(True)
             camera.lookAt(Vector(0, 100, 100))
-            self._camera_tool.rotateCam(0, 0)
+            self._camera_tool.rotateCam(0, 0) #type: ignore
 
         else:
             # for comparison is == used, because might not store them at the same location
@@ -393,6 +411,6 @@ class Controller:
             camera.lookAt(Vector(0, 100, 0))
 
             if coordinate == "x":
-                self._camera_tool.rotateCam(angle, 0)
+                self._camera_tool.rotateCam(angle, 0) #type: ignore
             elif coordinate == "y":
-                self._camera_tool.rotateCam(0, angle)
\ No newline at end of file
+                self._camera_tool.rotateCam(0, angle) #type: ignore
\ No newline at end of file
