@@ -1,48 +1,56 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal, pyqtSlot
+from PyQt5.QtQml import QQmlEngine, QJSEngine #For typing.
+from typing import List, Mapping, Optional, TYPE_CHECKING
 
 from UM.Application import Application
 from UM.i18n import i18nCatalog
-from UM.Scene.Selection import Selection
+from UM.Logger import Logger
 from UM.Message import Message
 from UM.OutputDevice import OutputDeviceError
-from UM.Logger import Logger
+import UM.Qt.QtApplication
+from UM.Scene.SceneNode import SceneNode
+from UM.Scene.Selection import Selection
+
+if TYPE_CHECKING:
+    from UM.FileHandler.FileHandler import FileHandler
+    from UM.OutputDevice.OutputDeviceManager import OutputDeviceManager
 
 catalog = i18nCatalog("uranium")
 
 class OutputDeviceManagerProxy(QObject):
-    def __init__(self, parent = None):
+    def __init__(self, parent = None) -> None:
         super().__init__(parent)
-        self._device_manager = Application.getInstance().getOutputDeviceManager()
+        self._device_manager = Application.getInstance().getOutputDeviceManager() #type: OutputDeviceManager
         self._device_manager.activeDeviceChanged.connect(self._onActiveDeviceChanged)
         self._onActiveDeviceChanged()
 
     activeDeviceChanged = pyqtSignal()
     
     @pyqtProperty(str, notify = activeDeviceChanged)
-    def activeDevice(self):
+    def activeDevice(self) -> str:
         return self._device_manager.getActiveDevice().getId()
 
     @pyqtSlot(str)
-    def setActiveDevice(self, device_id):
+    def setActiveDevice(self, device_id: str) -> None:
         self._device_manager.setActiveDevice(device_id)
 
     @pyqtProperty(str, notify = activeDeviceChanged)
-    def activeDeviceName(self):
+    def activeDeviceName(self) -> str:
         return self._device_manager.getActiveDevice().getName()
 
     @pyqtProperty(str, notify = activeDeviceChanged)
-    def activeDeviceIconName(self):
+    def activeDeviceIconName(self) -> str:
         return self._device_manager.getActiveDevice().getIconName()
 
     @pyqtProperty(str, notify = activeDeviceChanged)
-    def activeDeviceShortDescription(self):
+    def activeDeviceShortDescription(self) -> str:
         return self._device_manager.getActiveDevice().getShortDescription()
 
     @pyqtProperty(str, notify = activeDeviceChanged)
-    def activeDeviceDescription(self):
+    def activeDeviceDescription(self) -> str:
         return self._device_manager.getActiveDevice().getDescription()
 
     ##  Request that the current scene is written to the output device.
@@ -51,13 +59,13 @@ class OutputDeviceManagerProxy(QObject):
     #   A file format is chosen from the list of available file formats by the
     #   output device.
     #
-    #   \param device_id \type{string} The handle of the device to write to.
-    #   \param file_name \type{string} A suggestion for the file name to write
+    #   \param device_id The handle of the device to write to.
+    #   \param file_name A suggestion for the file name to write
     #   to. Can be freely ignored if providing a file name makes no sense.
     #   \param kwargs Keyword arguments:
     #       limit_mimetypes: Limit the possible mimetypes to use for writing to these types.
     @pyqtSlot(str, str, "QVariantMap")
-    def requestWriteToDevice(self, device_id, file_name, kwargs):
+    def requestWriteToDevice(self, device_id: str, file_name: str, kwargs: Mapping[str, str]) -> None:
         limit_mimetypes = kwargs.get("limit_mimetypes", None)
         file_type = kwargs.get("file_type", "mesh")
         preferred_mimetype = kwargs.get("preferred_mimetype", None)
@@ -72,13 +80,13 @@ class OutputDeviceManagerProxy(QObject):
     #   A file format is chosen from the list of available file formats by the
     #   output device.
     #
-    #   \param device_id \type{string} The handle of the device to write to.
-    #   \param file_name \type{string} A suggestion for the file name to write
+    #   \param device_id The handle of the device to write to.
+    #   \param file_name A suggestion for the file name to write
     #   to. Can be freely ignored if providing a file name makes no sense.
     #   \param kwargs Keyword arguments:
     #       limit_mimetypes: Limit the possible mimetypes to use for writing to these types.
     @pyqtSlot(str, str, "QVariantMap")
-    def requestWriteSelectionToDevice(self, device_id, file_name, kwargs):
+    def requestWriteSelectionToDevice(self, device_id: str, file_name: str, kwargs: Mapping[str, str]) -> None:
         if not Selection.hasSelection():
             return
 
@@ -89,7 +97,7 @@ class OutputDeviceManagerProxy(QObject):
         # loop, since that does work.
         Application.getInstance().callLater(self._writeToDevice, Selection.getAllSelectedObjects(), device_id, file_name, limit_mimetypes, preferred_mimetype = preferred_mimetype)
 
-    def _onActiveDeviceChanged(self):
+    def _onActiveDeviceChanged(self) -> None:
         self.activeDeviceChanged.emit()
 
     ##  Writes the specified node to the output device.
@@ -99,22 +107,20 @@ class OutputDeviceManagerProxy(QObject):
     #   output device.
     #
     #   \param nodes The scene nodes that must be written to the device.
-    #   \param device_id \type{string} The handle of the device to write to.
-    #   \param file_name \type{string} A suggestion for the file name to write
+    #   \param device_id The handle of the device to write to.
+    #   \param file_name A suggestion for the file name to write
     #   to. Can be freely ignored if providing a file name makes no sense.
     #   \param limit_mimetypes: Limit the possible mimetypes to use for writing to these types.
-    #   \param file_handler What file handler to get the writer from.
-    def _writeToDevice(self, nodes, device_id, file_name, limit_mimetypes, file_type = "mesh", **kwargs):
+    #   \param file_type What file handler to get the writer from.
+    def _writeToDevice(self, nodes: List[SceneNode], device_id: str, file_name: str, limit_mimetypes: bool, file_type: str = "mesh", **kwargs) -> None:
         device = self._device_manager.getOutputDevice(device_id)
         if not device:
             return
+        file_handler = None #type: Optional[FileHandler]
         if file_type == "mesh":
-            file_handler = Application.getInstance().getMeshFileHandler()
+            file_handler = UM.Qt.QtApplication.QtApplication.getInstance().getMeshFileHandler()
         elif file_type == "workspace":
-            file_handler = Application.getInstance().getWorkspaceFileHandler()
-        else:
-            # Unrecognised type
-            file_handler = None
+            file_handler = UM.Qt.QtApplication.QtApplication.getInstance().getWorkspaceFileHandler()
 
         try:
             device.requestWrite(nodes, file_name, limit_mimetypes, file_handler, **kwargs)
@@ -129,5 +135,5 @@ class OutputDeviceManagerProxy(QObject):
             Logger.logException("e", "Unable to write to file %s: %s", file_name, e)
 
 
-def createOutputDeviceManagerProxy(engine, script_engine):
+def createOutputDeviceManagerProxy(engine: QQmlEngine, script_engine: QJSEngine) -> OutputDeviceManagerProxy:
     return OutputDeviceManagerProxy()
