@@ -1,19 +1,21 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import sys
 import os
 import signal
-from typing import Dict, Optional
+from typing import cast, Dict, Optional, TYPE_CHECKING
 
 
 from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QUrl, pyqtProperty, pyqtSignal, pyqtSlot, QLocale, QTranslator, QLibraryInfo, QT_VERSION_STR, PYQT_VERSION_STR
 from PyQt5.QtQml import QQmlApplicationEngine, QQmlComponent, QQmlContext
 from PyQt5.QtWidgets import QApplication, QSplashScreen, QMessageBox, QSystemTrayIcon
-from PyQt5.QtGui import QGuiApplication, QIcon, QPixmap, QFontMetrics
+from PyQt5.QtGui import QIcon, QPixmap, QFontMetrics
 from PyQt5.QtCore import QTimer
 
 from UM.FileHandler.ReadFileJob import ReadFileJob
+from UM.Mesh.MeshFileHandler import MeshFileHandler
+from UM.Workspace.WorkspaceFileHandler import WorkspaceFileHandler
 from UM.Application import Application
 from UM.Qt.QtRenderer import QtRenderer
 from UM.Qt.Bindings.Bindings import Bindings
@@ -24,6 +26,12 @@ from UM.Preferences import Preferences
 from UM.i18n import i18nCatalog
 from UM.JobQueue import JobQueue
 from UM.View.GL.OpenGLContext import OpenGLContext
+from UM.Operations.GroupedOperation import GroupedOperation #To clear the scene.
+from UM.Operations.RemoveSceneNodeOperation import RemoveSceneNodeOperation #To clear the scene.
+from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator #To clear the scene.
+from UM.Scene.SceneNode import SceneNode #To clear the scene.
+from UM.Scene.Selection import Selection #To clear the selection after clearing the scene.
+from UM.Settings.ContainerRegistry import ContainerRegistry
 import UM.Settings.InstanceContainer  # For version upgrade to know the version number.
 import UM.Settings.ContainerStack  # For version upgrade to know the version number.
 import UM.Preferences  # For version upgrade to know the version number.
@@ -32,8 +40,7 @@ from UM.Mesh.ReadMeshJob import ReadMeshJob
 
 import UM.Qt.Bindings.Theme
 from UM.PluginRegistry import PluginRegistry
-MYPY = False
-if MYPY:
+if TYPE_CHECKING:
     from PyQt5.QtCore import QObject
 
 
@@ -73,8 +80,18 @@ class QtApplication(QApplication, Application):
 
         os.environ["QSG_RENDER_LOOP"] = "basic"
 
+        self._tray_icon_name = tray_icon_name
+
         super().__init__(sys.argv, **kwargs)
 
+        # For some reason, with Qt 5.9 and up, the default "windows" style seems like Windows 95. We have to set the
+        # style to "fusion" so it looks less ugly.
+        pyqt_version_parts = [int(n) for n in PYQT_VERSION_STR.split(".")]
+        if len(pyqt_version_parts) < 2:  # Make sure there are at less 2 parts in the version
+            pyqt_version_parts += [0 for _ in range(2 - len(pyqt_version_parts))]
+        if pyqt_version_parts[0] == 5 and pyqt_version_parts[1] > 8:
+            self.setStyle("fusion")
+
         self.setAttribute(Qt.AA_UseDesktopOpenGL)
         major_version, minor_version, profile = OpenGLContext.detectBestOpenGLVersion()
 
@@ -107,23 +124,39 @@ class QtApplication(QApplication, Application):
         # This is done here as a lot of plugins require a correct gl context. If you want to change the framework,
         # these checks need to be done in your <framework>Application.py class __init__().
 
+        self._mesh_file_handler = MeshFileHandler.getInstance() #type: MeshFileHandler
+        self._mesh_file_handler.setApplication(self)
+        self._workspace_file_handler = WorkspaceFileHandler.getInstance() #type: WorkspaceFileHandler
+        self._workspace_file_handler.setApplication(self)
+
+    def initialize(self):
         i18n_catalog = i18nCatalog("uranium")
 
         self.showSplashMessage(i18n_catalog.i18nc("@info:progress", "Loading plugins..."))
+        # Remove and install the plugins that have been scheduled
+        self._plugin_registry.initializeBeforePluginsAreLoaded()
         self._loadPlugins()
+        self._plugin_registry.initializeAfterPluginsAreLoaded()
         self._plugin_registry.checkRequiredPlugins(self.getRequiredPlugins())
         self.pluginsLoaded.emit()
 
         self.showSplashMessage(i18n_catalog.i18nc("@info:progress", "Updating configuration..."))
-        upgraded = UM.VersionUpgradeManager.VersionUpgradeManager.getInstance().upgrade()
-        if upgraded:
-            # Preferences might have changed. Load them again.
-            # Note that the language can't be updated, so that will always revert to English.
-            preferences = Preferences.getInstance()
-            try:
-                preferences.readFromFile(Resources.getPath(Resources.Preferences, self._application_name + ".cfg"))
-            except FileNotFoundError:
-                pass
+        with ContainerRegistry.getInstance().lockFile():
+            UM.VersionUpgradeManager.VersionUpgradeManager.getInstance().upgrade()
+
+        # Preferences might have changed. Load them again.
+        # Note that the language can't be updated, so that will always revert to English.
+        preferences = Preferences.getInstance()
+        try:
+            file_name = Resources.getPath(Resources.Preferences, self._application_name + ".cfg")
+            with open(file_name, "r", encoding = "utf-8") as f:
+                serialized = f.read()
+            preferences.deserialize(serialized)
+        except FileNotFoundError:
+            pass
+        # Force the configuration file to be written again since the list of plugins to remove maybe changed
+        Preferences.getInstance().setValue("general/plugins_to_remove", "")
+        Preferences.getInstance().writeToFile(Resources.getStoragePath(Resources.Preferences, self.getApplicationName() + ".cfg"))
 
         self.showSplashMessage(i18n_catalog.i18nc("@info:progress", "Loading preferences..."))
         try:
@@ -149,8 +182,8 @@ class QtApplication(QApplication, Application):
         # Initialize System tray icon and make it invisible because it is used only to show pop up messages
         self._tray_icon = None
         self._tray_icon_widget = None
-        if tray_icon_name:
-            self._tray_icon = QIcon(Resources.getPath(Resources.Images, tray_icon_name))
+        if self._tray_icon_name:
+            self._tray_icon = QIcon(Resources.getPath(Resources.Images, self._tray_icon_name))
             self._tray_icon_widget = QSystemTrayIcon(self._tray_icon)
             self._tray_icon_widget.setVisible(False)
 
@@ -185,6 +218,7 @@ class QtApplication(QApplication, Application):
     def hideMessage(self, message):
         with self._message_lock:
             if message in self._visible_messages:
+                message.hide(send_signal = False)  # we're in handling hideMessageSignal so we don't want to resend it
                 self._visible_messages.remove(message)
                 self.visibleMessageRemoved.emit(message)
 
@@ -341,14 +375,16 @@ class QtApplication(QApplication, Application):
 
         return super().event(event)
 
-    def windowClosed(self):
+    def windowClosed(self, save_data: bool = True) -> None:
         Logger.log("d", "Shutting down %s", self.getApplicationName())
         self._shutting_down = True
 
-        try:
-            Preferences.getInstance().writeToFile(Resources.getStoragePath(Resources.Preferences, self.getApplicationName() + ".cfg"))
-        except Exception as e:
-            Logger.log("e", "Exception while saving preferences: %s", repr(e))
+        if save_data:
+            try:
+                Preferences.getInstance().writeToFile(Resources.getStoragePath(Resources.Preferences,
+                                                                               self.getApplicationName() + ".cfg"))
+            except Exception as e:
+                Logger.log("e", "Exception while saving preferences: %s", repr(e))
 
         try:
             self.applicationShuttingDown.emit()
@@ -477,12 +513,60 @@ class QtApplication(QApplication, Application):
             Logger.log("e", str(err.toString()))
         if result is None:
             return None
-        
+
         # We need to store the context with the qml object, else the context gets garbage collected and the qml objects
         # no longer function correctly/application crashes.
         result.attached_context = result_context
         return result
 
+    ##  Delete all nodes containing mesh data in the scene.
+    #   \param only_selectable. Set this to False to delete objects from all build plates
+    @pyqtSlot()
+    def deleteAll(self, only_selectable = True) -> None:
+        Logger.log("i", "Clearing scene")
+        if not self.getController().getToolsEnabled():
+            return
+
+        nodes = []
+        for node in DepthFirstIterator(self.getController().getScene().getRoot()): #type: ignore #Ignore type error because iter() should get called automatically by Python syntax.
+            if not isinstance(node, SceneNode):
+                continue
+            if (not node.getMeshData() and not node.callDecoration("getLayerData")) and not node.callDecoration("isGroup"):
+                continue  # Node that doesnt have a mesh and is not a group.
+            if only_selectable and not node.isSelectable():
+                continue
+            if not node.callDecoration("isSliceable") and not node.callDecoration("getLayerData") and not node.callDecoration("isGroup"):
+                continue  # Only remove nodes that are selectable.
+            if node.getParent() and node.getParent().callDecoration("isGroup"):
+                continue  # Grouped nodes don't need resetting as their parent (the group) is resetted)
+            nodes.append(node)
+        if nodes:
+            op = GroupedOperation()
+
+            for node in nodes:
+                op.addOperation(RemoveSceneNodeOperation(node))
+
+                # Reset the print information
+                self.getController().getScene().sceneChanged.emit(node)
+
+            op.push()
+            Selection.clear()
+
+    ##  Get the MeshFileHandler of this application.
+    def getMeshFileHandler(self) -> MeshFileHandler:
+        return self._mesh_file_handler
+
+    def getWorkspaceFileHandler(self) -> WorkspaceFileHandler:
+        return self._workspace_file_handler
+
+    ##  Gets the instance of this application.
+    #
+    #   This is just to further specify the type of Application.getInstance().
+    #   \return The instance of this application.
+    @classmethod
+    def getInstance(cls, **kwargs) -> "QtApplication":
+        return cast(QtApplication, super().getInstance(**kwargs))
+
     def _createSplashScreen(self):
         return QSplashScreen(QPixmap(Resources.getPath(Resources.Images, self.getApplicationName() + ".png")))
 
