@@ -1,21 +1,25 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import configparser
 import io
-from typing import Any, cast, Dict, List, Optional, Set
+from typing import Any, cast, Dict, List, Optional, Set, Tuple
 
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal
+from PyQt5.QtQml import QQmlEngine
 import UM.FlameProfiler
 
-from UM.Settings.SettingDefinition import SettingDefinition
+from UM.ConfigurationErrorMessage import ConfigurationErrorMessage
 from UM.Signal import Signal, signalemitter
 from UM.PluginObject import PluginObject
 from UM.Logger import Logger
 from UM.MimeTypeDatabase import MimeTypeDatabase, MimeType
+from UM.Settings.ContainerFormatError import ContainerFormatError
+import UM.Settings.ContainerRegistry
 from UM.Settings.DefinitionContainer import DefinitionContainer #For getting all definitions in this stack.
 from UM.Settings.Interfaces import ContainerInterface, ContainerRegistryInterface
 from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext
+from UM.Settings.SettingDefinition import SettingDefinition
 from UM.Settings.SettingFunction import SettingFunction
 from UM.Settings.Validator import ValidatorState
 
@@ -47,49 +51,48 @@ MimeTypeDatabase.addMimeType(
 ##  A stack of setting containers to handle setting value retrieval.
 @signalemitter
 class ContainerStack(QObject, ContainerInterface, PluginObject):
-    Version = 3 # type: int
+    Version = 4 # type: int
 
     ##  Constructor
     #
-    #   \param stack_id \type{string} A unique, machine readable/writable ID.
-    def __init__(self, stack_id: str, *args, **kwargs):
-        # Note that we explicitly pass None as QObject parent here. This is to be able
-        # to support pickling.
-        super().__init__(parent = None, *args, **kwargs)
+    #   \param stack_id A unique, machine readable/writable ID.
+    def __init__(self, stack_id: str) -> None:
+        super().__init__()
+        QQmlEngine.setObjectOwnership(self, QQmlEngine.CppOwnership)
 
         self._metadata = {
             "id": stack_id,
             "name": stack_id,
             "version": self.Version,
             "container_type": ContainerStack
-        }
+        } #type: Dict[str, Any]
         self._containers = []  # type: List[ContainerInterface]
         self._next_stack = None  # type: Optional[ContainerStack]
         self._read_only = False  # type: bool
         self._dirty = True  # type: bool
         self._path = ""  # type: str
-        self._postponed_emits = []  # gets filled with 2-tuples: signal, signal_argument(s)
+        self._postponed_emits = [] #type: List[Tuple[Signal, ContainerInterface]] # gets filled with 2-tuples: signal, signal_argument(s)
 
-        self._property_changes = {}
+        self._property_changes = {} #type: Dict[str, Set[str]]
         self._emit_property_changed_queued = False  # type: bool
 
     ##  For pickle support
-    def __getnewargs__(self):
+    def __getnewargs__(self) -> Tuple[str]:
         return (self.getId(),)
 
     ##  For pickle support
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         return self.__dict__
 
     ##  For pickle support
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         self.__dict__.update(state)
 
     ##  \copydoc ContainerInterface::getId
     #
     #   Reimplemented from ContainerInterface
     def getId(self) -> str:
-        return self._metadata["id"]
+        return cast(str, self._metadata["id"])
 
     id = pyqtProperty(str, fget = getId, constant = True)
 
@@ -106,6 +109,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         if name != self.getName():
             self._metadata["name"] = name
             self.nameChanged.emit()
+            self.metaDataChanged.emit(self)
 
     ##  Emitted whenever the name of this stack changes.
     nameChanged = pyqtSignal()
@@ -117,7 +121,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     def isReadOnly(self) -> bool:
         return self._read_only
 
-    def setReadOnly(self, read_only):
+    def setReadOnly(self, read_only: bool) -> None:
         if read_only != self._read_only:
             self._read_only = read_only
             self.readOnlyChanged.emit()
@@ -128,13 +132,13 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getMetaData
     #
     #   Reimplemented from ContainerInterface
-    def getMetaData(self):
+    def getMetaData(self) -> Dict[str, Any]:
         return self._metadata
 
     ##  Set the complete set of metadata
-    def setMetaData(self, meta_data):
-        if meta_data != self._meta_data:
-            self._meta_data = meta_data
+    def setMetaData(self, meta_data: Dict[str, Any]) -> None:
+        if meta_data != self.getMetaData():
+            self._metadata = meta_data
             self.metaDataChanged.emit(self)
 
     metaDataChanged = pyqtSignal(QObject)
@@ -143,7 +147,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getMetaDataEntry
     #
     #   Reimplemented from ContainerInterface
-    def getMetaDataEntry(self, entry: str, default = None):
+    def getMetaDataEntry(self, entry: str, default = None) -> Any:
         value = self._metadata.get(entry, None)
 
         if value is None:
@@ -157,7 +161,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         else:
             return value
 
-    def addMetaDataEntry(self, key: str, value):
+    def addMetaDataEntry(self, key: str, value: Any) -> None:
         if key not in self._metadata:
             self._dirty = True
             self._metadata[key] = value
@@ -165,7 +169,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         else:
             Logger.log("w", "Meta data with key %s was already added.", key)
 
-    def setMetaDataEntry(self, key, value):
+    def setMetaDataEntry(self, key: str, value: Any) -> None:
         if key in self._metadata:
             self._dirty = True
             self._metadata[key] = value
@@ -173,7 +177,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         else:
             Logger.log("w", "Meta data with key %s was not found. Unable to change.", key)
 
-    def removeMetaDataEntry(self, key):
+    def removeMetaDataEntry(self, key: str) -> None:
         if key in self._metadata:
             del self._metadata[key]
             self.metaDataChanged.emit(self)
@@ -199,7 +203,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   Note that if the property value is a function, this method will return the
     #   result of evaluating that property with the current stack. If you need the
     #   actual function, use getRawProperty()
-    def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None):
+    def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None) -> Any:
         value = self.getRawProperty(key, property_name, context = context)
         if isinstance(value, SettingFunction):
             if context is not None:
@@ -226,8 +230,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \return The raw property value of the property, or None if not found. Note that
     #           the value might be a SettingFunction instance.
     #
-    def getRawProperty(self, key, property_name, *, context: Optional[PropertyEvaluationContext] = None,
-                       use_next = True, skip_until_container = None):
+    def getRawProperty(self, key: str, property_name: str, *, context: Optional[PropertyEvaluationContext] = None,
+                       use_next: bool = True, skip_until_container: Optional[ContainerInterface] = None) -> Any:
         containers = self._containers
         if context is not None:
             # if context is provided, check if there is any container that needs to be skipped.
@@ -278,7 +282,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   Reimplemented from ContainerInterface
     #
     #   TODO: Expand documentation here, include the fact that this should _not_ include all containers
-    def serialize(self, ignored_metadata_keys: Optional[set] = None):
+    def serialize(self, ignored_metadata_keys: Optional[set] = None) -> str:
         parser = configparser.ConfigParser(interpolation = None, empty_lines_in_values = False)
 
         parser["general"] = {}
@@ -323,6 +327,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         try:
             parser = cls._readAndValidateSerialized(serialized)
             configuration_type = parser["metadata"]["type"]
+        except InvalidContainerStackError as icse:
+            raise icse
         except Exception as e:
             Logger.log("e", "Could not get configuration type: %s", e)
         return configuration_type
@@ -330,6 +336,10 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     @classmethod
     def getVersionFromSerialized(cls, serialized: str) -> Optional[int]:
         configuration_type = cls.getConfigurationTypeFromSerialized(serialized)
+        if not configuration_type:
+            Logger.log("d", "Could not get type from serialized.")
+            return None
+
         # get version
         version = None
         try:
@@ -351,7 +361,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         parser = self._readAndValidateSerialized(serialized)
 
         if parser["general"].getint("version") != self.Version:
-            raise IncorrectVersionError
+            raise IncorrectVersionError()
 
         # Clear all data before starting.
         for container in self._containers:
@@ -374,12 +384,15 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                     containers[0].propertyChanged.connect(self._collectPropertyChanges)
                     self._containers.append(containers[0])
                 else:
-                    raise Exception("When trying to deserialize %s, we received an unknown ID (%s) for container" % (self.getId(), container_id))
+                    self._containers.append(_containerRegistry.getEmptyInstanceContainer())
+                    ConfigurationErrorMessage.getInstance().addFaultyContainers(container_id, self.getId())
+                    Logger.log("e", "When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
+                    raise ContainerFormatError("When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
 
         elif parser.has_option("general", "containers"):
             # Backward compatibility with 2.3.1: The containers used to be saved in a single comma-separated list.
             container_string = parser["general"].get("containers", "")
-            Logger.log("d", "While deserializeing, we got the following container string: %s", container_string)
+            Logger.log("d", "While deserializing, we got the following container string: %s", container_string)
             container_id_list = container_string.split(",")
             for container_id in container_id_list:
                 if container_id != "":
@@ -388,7 +401,10 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                         containers[0].propertyChanged.connect(self._collectPropertyChanges)
                         self._containers.append(containers[0])
                     else:
-                        raise Exception("When trying to deserialize %s, we received an unknown ID (%s) for container" % (self.getId(), container_id))
+                        self._containers.append(_containerRegistry.getEmptyInstanceContainer())
+                        ConfigurationErrorMessage.getInstance().addFaultyContainers(container_id, self.getId())
+                        Logger.log("e", "When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
+                        raise ContainerFormatError("When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
 
         ## TODO; Deserialize the containers.
 
@@ -418,8 +434,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         try:
             metadata["name"] = parser["general"]["name"]
             metadata["version"] = parser["general"]["version"]
-        except KeyError: #One of the keys or the General section itself is missing.
-            return []
+        except KeyError as e: #One of the keys or the General section itself is missing.
+            raise InvalidContainerStackError("Missing required fields: {error_msg}".format(error_msg = str(e)))
 
         if "metadata" in parser:
             metadata.update(parser["metadata"])
@@ -495,11 +511,11 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::setPath
     #
     #   Reimplemented from ContainerInterface
-    def setPath(self, path: str):
+    def setPath(self, path: str) -> None:
         self._path = path
 
     ##  Get the SettingDefinition object for a specified key
-    def getSettingDefinition(self, key: str):
+    def getSettingDefinition(self, key: str) -> Optional[SettingDefinition]:
         for container in self._containers:
             if not isinstance(container, DefinitionContainer):
                 continue
@@ -515,15 +531,14 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
 
     ##  Find a container matching certain criteria.
     #
-    #   \param filter \type{dict} A dictionary containing key and value pairs
-    #   that need to match the container. Note that the value of "*" can be used
-    #   as a wild card. This will ensure that any container that has the
-    #   specified key in the meta data is found.
-    #   \param container_type \type{class} An optional type of container to
-    #   filter on.
+    #   \param criteria A dictionary containing key and value pairs that need to
+    #   match the container. Note that the value of "*" can be used as a wild
+    #   card. This will ensure that any container that has the specified key in
+    #   the meta data is found.
+    #   \param container_type An optional type of container to filter on.
     #   \return The first container that matches the filter criteria or None if not found.
     @UM.FlameProfiler.profile
-    def findContainer(self, criteria = None, container_type = None, **kwargs) -> Optional[ContainerInterface]:
+    def findContainer(self, criteria: Dict[str, Any] = None, container_type: type = None, **kwargs: Any) -> Optional[ContainerInterface]:
         if not criteria and kwargs:
             criteria = kwargs
         elif criteria is None:
@@ -553,15 +568,15 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  Add a container to the top of the stack.
     #
     #   \param container The container to add to the stack.
-    def addContainer(self, container):
+    def addContainer(self, container: ContainerInterface) -> None:
         self.insertContainer(0, container)
 
     ##  Insert a container into the stack.
     #
-    #   \param index \type{int} The index of to insert the container at.
+    #   \param index The index of to insert the container at.
     #          A negative index counts from the bottom
     #   \param container The container to add to the stack.
-    def insertContainer(self, index, container):
+    def insertContainer(self, index: int, container: ContainerInterface) -> None:
         if container is self:
             raise Exception("Unable to add stack to itself.")
 
@@ -577,7 +592,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #
     #   \exception IndexError Raised when the specified index is out of bounds.
     #   \exception Exception when trying to replace container ContainerStack.
-    def replaceContainer(self, index: int, container: ContainerInterface, postpone_emit=False):
+    def replaceContainer(self, index: int, container: ContainerInterface, postpone_emit: bool = False) -> None:
         if index < 0:
             raise IndexError
         if container is self:
@@ -597,7 +612,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \param index \type{int} The index of the container to remove.
     #
     #   \exception IndexError Raised when the specified index is out of bounds.
-    def removeContainer(self, index: int = 0):
+    def removeContainer(self, index: int = 0) -> None:
         if index < 0:
             raise IndexError
         try:
@@ -622,7 +637,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \param stack \type{ContainerStack} The next stack to set. Can be None.
     #   Raises Exception when trying to set itself as next stack (to prevent infinite loops)
     #   \sa getNextStack
-    def setNextStack(self, stack: "ContainerStack", connect_signals: bool = True):
+    def setNextStack(self, stack: "ContainerStack", connect_signals: bool = True) -> None:
         if self is stack:
             raise Exception("Next stack can not be itself")
         if self._next_stack == stack:
@@ -630,14 +645,16 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
 
         if self._next_stack:
             self._next_stack.propertyChanged.disconnect(self._collectPropertyChanges)
+            self.containersChanged.disconnect(self._next_stack.containersChanged)
         self._next_stack = stack
         if self._next_stack and connect_signals:
             self._next_stack.propertyChanged.connect(self._collectPropertyChanges)
+            self.containersChanged.connect(self._next_stack.containersChanged)
 
     ##  Send postponed emits
     #   These emits are collected from the option postpone_emit.
     #   Note: the option can be implemented for all functions modifying the stack.
-    def sendPostponedEmits(self):
+    def sendPostponedEmits(self) -> None:
         while self._postponed_emits:
             signal, signal_arg = self._postponed_emits.pop(0)
             signal.emit(signal_arg)
@@ -654,7 +671,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 # Setting is not validated. This can happen if there is only a setting definition.
                 # We do need to validate it, because a setting defintions value can be set by a function, which could
                 # be an invalid setting.
-                definition = self.getSettingDefinition(key)
+                definition = cast(SettingDefinition, self.getSettingDefinition(key))
                 validator_type = SettingDefinition.getValidatorForType(definition.type)
                 if validator_type:
                     validator = validator_type(key)
@@ -673,7 +690,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 # Setting is not validated. This can happen if there is only a setting definition.
                 # We do need to validate it, because a setting defintions value can be set by a function, which could
                 # be an invalid setting.
-                definition = self.getSettingDefinition(key)
+                definition = cast(SettingDefinition, self.getSettingDefinition(key))
                 validator_type = SettingDefinition.getValidatorForType(definition.type)
                 if validator_type:
                     validator = validator_type(key)
@@ -688,7 +705,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     # loop can run. This prevents us from sending the same change signal multiple times.
     # In addition, it allows us to emit a single signal that reports all properties that
     # have changed.
-    def _collectPropertyChanges(self, key: str, property_name: str):
+    def _collectPropertyChanges(self, key: str, property_name: str) -> None:
         if key not in self._property_changes:
             self._property_changes[key] = set()
 
@@ -699,7 +716,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
             self._emit_property_changed_queued = True
 
     # Perform the emission of the change signals that were collected in a previous step.
-    def _emitCollectedPropertyChanges(self):
+    def _emitCollectedPropertyChanges(self) -> None:
         for key, property_names in self._property_changes.items():
             self.propertiesChanged.emit(key, property_names)
 
@@ -709,7 +726,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         self._property_changes = {}
         self._emit_property_changed_queued = False
 
-_containerRegistry = None   # type:  ContainerRegistryInterface
+_containerRegistry = ContainerRegistryInterface() # type: ContainerRegistryInterface
 
 def setContainerRegistry(registry: ContainerRegistryInterface) -> None:
     global _containerRegistry
