@@ -1,22 +1,26 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import sys
 import ctypes   # type: ignore
 
-from PyQt5.QtGui import QOpenGLVersionProfile, QOpenGLContext, QOpenGLFramebufferObject, QOpenGLBuffer, QSurfaceFormat
+from PyQt5.QtGui import QOpenGLVersionProfile, QOpenGLContext, QOpenGLFramebufferObject, QOpenGLBuffer
 from PyQt5.QtWidgets import QMessageBox
+from typing import Any, TYPE_CHECKING
 
 from UM.Logger import Logger
 
-from UM.View.GL import FrameBufferObject
-from UM.View.GL import ShaderProgram
+from UM.View.GL.FrameBufferObject import FrameBufferObject
+from UM.View.GL.ShaderProgram import ShaderProgram
 from UM.View.GL.ShaderProgram import InvalidShaderProgramError
-from UM.View.GL import Texture
+from UM.View.GL.Texture import Texture
 from UM.View.GL.OpenGLContext import OpenGLContext
-from UM.i18n import i18nCatalog #To make dialogs translateable.
+from UM.i18n import i18nCatalog  # To make dialogs translatable.
 i18n_catalog = i18nCatalog("uranium")
 
+if TYPE_CHECKING:
+    from UM.Mesh.MeshData import MeshData
+
 ##  Convenience methods for dealing with OpenGL.
 #
 #   This class simplifies dealing with OpenGL and different Python OpenGL bindings. It
@@ -24,7 +28,7 @@ i18n_catalog = i18nCatalog("uranium")
 #   functionality using these different OpenGL bindings. Additionally, it provides singleton
 #   handling. The implementation-defined subclass must be set as singleton instance as soon
 #   as possible so that any calls to getInstance() return a proper object.
-class OpenGL(object):
+class OpenGL:
     VertexBufferProperty = "__vertex_buffer"
     IndexBufferProperty = "__index_buffer"
 
@@ -35,22 +39,22 @@ class OpenGL(object):
         Intel = 3
         Other = 4
 
-    def __init__(self):
+    def __init__(self) -> None:
         profile = QOpenGLVersionProfile()
         profile.setVersion(OpenGLContext.major_version, OpenGLContext.minor_version)
         profile.setProfile(OpenGLContext.profile)
 
-        self._gl = QOpenGLContext.currentContext().versionFunctions(profile)
+        self._gl = QOpenGLContext.currentContext().versionFunctions(profile) #Don't add a type here! It's actually a protected class in PyQt that depends on the implementation of your graphics card.
         if not self._gl:
             Logger.log("e", "Startup failed due to OpenGL initialization failing")
             QMessageBox.critical(None, i18n_catalog.i18nc("@message", "Failed to Initialize OpenGL", "Could not initialize OpenGL. This program requires OpenGL 2.0 or higher. Please check your video card drivers."))
             sys.exit(1)
 
-        # It would be nice to be able to not necessarily need OpenGL Framebuffer Object support, but
-        # due to a limiation in PyQt, currently glReadPixels or similar methods are not available.
+        # It would be nice to be able to not necessarily need OpenGL FrameBuffer Object support, but
+        # due to a limitation in PyQt, currently glReadPixels or similar methods are not available.
         # This means we can only get frame buffer contents through methods that indirectly call
-        # those methods, in this case primarily QOpenGLFramebufferObject::toImage(), making us
-        # hard-depend on Framebuffer Objects.
+        # those methods, in this case primarily QOpenGLFrameBufferObject::toImage(), making us
+        # hard-depend on FrameBuffer Objects.
         if not self.hasFrameBufferObjects():
             Logger.log("e", "Starup failed, OpenGL does not support Frame Buffer Objects")
             QMessageBox.critical(None, i18n_catalog.i18nc("Critical OpenGL Extensions Missing", "Critical OpenGL extensions are missing. This program requires support for Framebuffer Objects. Please check your video card drivers."))
@@ -58,11 +62,12 @@ class OpenGL(object):
 
         self._gl.initializeOpenGLFunctions()
 
-        self._gpu_vendor = OpenGL.Vendor.Other
+        self._gpu_vendor = OpenGL.Vendor.Other #type: int
         vendor_string = self._gl.glGetString(self._gl.GL_VENDOR)
         if vendor_string is None:
             vendor_string = "Unknown"
         vendor_string = vendor_string.lower()
+
         if "nvidia" in vendor_string:
             self._gpu_vendor = OpenGL.Vendor.NVidia
         elif "amd" in vendor_string or "ati" in vendor_string:
@@ -70,39 +75,53 @@ class OpenGL(object):
         elif "intel" in vendor_string:
             self._gpu_vendor = OpenGL.Vendor.Intel
 
-        #WORKAROUND: Cura/#1117 Cura-packaging/12
+        # WORKAROUND: Cura/#1117 Cura-packaging/12
         # Some Intel GPU chipsets return a string, which is not undecodable via PyQt5.
         # This workaround makes the code fall back to a "Unknown" renderer in these cases.
         try:
-            self._gpu_type = self._gl.glGetString(self._gl.GL_RENDERER)
+            self._gpu_type = self._gl.glGetString(self._gl.GL_RENDERER) #type: str
         except UnicodeDecodeError:
             Logger.log("e", "DecodeError while getting GL_RENDERER via glGetString!")
-            self._gpu_type = "Unknown"
+            self._gpu_type = "Unknown" #type: str
+
+        self._opengl_version = self._gl.glGetString(self._gl.GL_VERSION) #type: str
 
         if not self.hasFrameBufferObjects():
             Logger.log("w", "No frame buffer support, falling back to texture copies.")
 
         Logger.log("d", "Initialized OpenGL subsystems.")
-        Logger.log("d", "OpenGL Version:  %s", self._gl.glGetString(self._gl.GL_VERSION))
+        Logger.log("d", "OpenGL Version:  %s", self._opengl_version)
         Logger.log("d", "OpenGL Vendor:   %s", self._gl.glGetString(self._gl.GL_VENDOR))
         Logger.log("d", "OpenGL Renderer: %s", self._gpu_type)
 
     ##  Check if the current OpenGL implementation supports FrameBuffer Objects.
     #
     #   \return True if FBOs are supported, False if not.
-    def hasFrameBufferObjects(self):
+    def hasFrameBufferObjects(self) -> bool:
         return QOpenGLFramebufferObject.hasOpenGLFramebufferObjects()
 
+    ##  Get the current OpenGL version.
+    #
+    #   \return Version of OpenGL
+    def getOpenGLVersion(self) -> str:
+        return self._opengl_version
+
+    ##  Get the current GPU vendor name.
+    #
+    #   \return Name of the vendor of current GPU
+    def getGPUVendorName(self) -> str:
+        return self._gl.glGetString(self._gl.GL_VENDOR)
+
     ##  Get the current GPU vendor.
     #
     #   \return One of the items of OpenGL.Vendor.
-    def getGPUVendor(self):
+    def getGPUVendor(self) -> int:
         return self._gpu_vendor
 
     ##  Get a string describing the current GPU type.
     #
     #   This effectively should return the OpenGL renderer string.
-    def getGPUType(self):
+    def getGPUType(self) -> str:
         return self._gpu_type
 
     ##  Get the OpenGL bindings object.
@@ -117,20 +136,20 @@ class OpenGL(object):
     ##  Create a FrameBuffer Object.
     #
     #   This should return an implementation-specifc FrameBufferObject subclass.
-    def createFrameBufferObject(self, width, height):
-        return FrameBufferObject.FrameBufferObject(width, height)
+    def createFrameBufferObject(self, width: int, height: int) -> FrameBufferObject:
+        return FrameBufferObject(width, height)
 
     ##  Create a Texture Object.
     #
-    #   This should return an implementation-specifc Texture subclass.
-    def createTexture(self):
-        return Texture.Texture(self._gl)
+    #   This should return an implementation-specific Texture subclass.
+    def createTexture(self) -> Texture:
+        return Texture(self._gl)
 
     ##  Create a ShaderProgram Object.
     #
     #   This should return an implementation-specifc ShaderProgram subclass.
-    def createShaderProgram(self, file_name):
-        shader = ShaderProgram.ShaderProgram()
+    def createShaderProgram(self, file_name: str) -> ShaderProgram:
+        shader = ShaderProgram()
         # The version_string must match the keys in shader files.
         if OpenGLContext.isLegacyOpenGL():
             version_string = ""  # Nothing is added to "fragment" and "vertex"
@@ -138,7 +157,7 @@ class OpenGL(object):
             version_string = "41core"
 
         try:
-            shader.load(file_name, version=version_string)
+            shader.load(file_name, version = version_string)
         except InvalidShaderProgramError:
             # If the loading failed, it could be that there is no specific shader for this version.
             # Try again without a version nr to get the generic one.
@@ -159,7 +178,7 @@ class OpenGL(object):
     #   \param kwargs Keyword arguments.
     #                 Possible values:
     #                 - force_recreate: Ignore the cached value if set and always create a new buffer.
-    def createVertexBuffer(self, mesh, **kwargs):
+    def createVertexBuffer(self, mesh: "MeshData", **kwargs: Any) -> QOpenGLBuffer:
         if not kwargs.get("force_recreate", False) and hasattr(mesh, OpenGL.VertexBufferProperty):
             return getattr(mesh, OpenGL.VertexBufferProperty)
 
@@ -237,7 +256,7 @@ class OpenGL(object):
     #   \param kwargs Keyword arguments.
     #                 Possible values:
     #                 - force_recreate: Ignore the cached value if set and always create a new buffer.
-    def createIndexBuffer(self, mesh, **kwargs):
+    def createIndexBuffer(self, mesh: "MeshData", **kwargs: Any):
         if not mesh.hasIndices():
             return None
 
@@ -271,7 +290,7 @@ class OpenGL(object):
     #   This is mostly meant to simplify the singleton logic and should be called
     #   by the OpenGL implementation as soon as possible.
     @classmethod
-    def setInstance(cls, instance):
+    def setInstance(cls, instance: "OpenGL") -> None:
         cls._instance = instance
 
     ## private:
